function [AE,BE,CE,DE,QE,RE,init_xE,init_PE,k,logvec,xfilt]=em_algorithm(y,u,A,B,C,D,Q,R,init_x,init_P,Tol,Nmax);

%This function estimates the matrices in the state space model AE, BE, CE,
%DE, QE, RE, init_xE, init_PE from initial guess values.

% x_{t+1} = A*x_{t} + B*u_{t} + w_{t}   var{w_(t)} = Q
% y_{t}   = C*x_{t} + D*u_{t} + v_{t}   var{v_(t)} = R

%In this function A, B, init_x will be known 
%We are going only to estimate Q, R, C, D.
%k is the number of iterations until convergence. 
%Nmax is the maximum number of iterations allowed in the algorithm.
%logvec is the likelihood value in all the iterations of the EM algorithm.
%xfilt are the estimation of the state vector x_{t} knowing the values until y_{t-1} via Kalman filter.
%Tol is the stop criterion of the EM algorithm


T=size(y,2);
ss=size(A,1);
ne=size(u,1);

AE=A;
BE=B;
CE=C;
DE=D;
QE=Q;
RE=R;
init_xE=init_x;
init_PE=init_P;  
[LQE,a]=eig(QE);
i=find(((diag(a)<=0) | (imag(diag(a))~=0)));
for j=1:length(i)
    a(i(j),i(j))=0;
end
QE=LQE*a*LQE';

z=zeros(ss,1);
z(:,1)=eig(AE);

[LRE,a]=eig(RE);
i=find(((diag(a)<=0) | (imag(diag(a))~=0)));
for j=1:length(i)
    a(i(j),i(j))=0;
end
RE=LRE*a*LRE';


Suue=u(:,1)*u(:,1)'; %Sum((u_t-1)*(u_t-1)') for t=2:N
Syu=y(:,2)*u(:,2)'; %Sum((y_t)*(u_t)') for t=2:N
Syy=y(:,2)*y(:,2)'; %Sum((y_t)*(y_t)') for t=2:N
Suuo=u(:,2)*u(:,2)'; %Sum((u_t)*(u_t)') for t=2:N
for t=3:T
    Suue=Suue+u(:,t-1)*u(:,t-1)';
    Syu=Syu+y(:,t)*u(:,t)'; 
    Syy=Syy+y(:,t)*y(:,t)'; 
    Suuo=Suuo+u(:,t)*u(:,t)';
end
Suy=Syu';

k=0;
loglik_nueva=1e+7;
while( k<2 || ((abs(loglik_nueva-loglik_antigua)/abs(loglik_nueva))>Tol & k<Nmax) ) 
    
    %Step 1: Expectation   
  %------------------------------------
    [init_xE,init_PE,Sub,Suxe,Sbb,Sbx,Sxx,Sxy,Suxo,loglik,xfilt]=kalman_smoother2s(y,u,AE,BE,CE,DE,QE,RE,init_xE,init_PE);
   
    
    
    Sbu=Sub';
    Sxue=Suxe';
    Sxb=Sbx';
    Syx=Sxy';
    Sxuo=Suxo';
           
    loglik_antigua=loglik_nueva;
    loglik_nueva=loglik; 
    logvec(k+1)=loglik;
    
    %Step 2: Maximization
  %-------------------------------------
  
    %Estimation of CE and DE.
    if (sum(u(:))>1)
        %With inputs
        SZU=[Syx Syu]; SZZ=[Sxx Sxuo; Suxo Suuo];
        CDE=SZU/SZZ; 
        CE=CDE(:,1:ss);
        DE=CDE(:,ss+1:ss+ne);
        %DE(:,1)=DE(:,1)+CE*init_xE;
        init_xE=init_x; %Initial state of value zero.        
    else
        %Without inputs
        CE=(Syx-D*Suxo)/Sxx;
        DE=D; %D is known equal to zero.
    end
       
    
    %Estimation of QE
    QE=(Sxx-Sxb*AE'-AE*Sbx+AE*Sbb*AE'-Sxue*BE'+AE*Sbu*BE'-BE*Suxe+BE*Sub*AE'+BE*Suue*BE')/(T-1);
      
    [LQE,a]=eig(QE);
    i=find(((diag(a)<=0) | (imag(diag(a))~=0)));
    for j=1:length(i)
        a(i(j),i(j))=0;
    end
    QE=LQE*a*LQE';
  
    
    %Estimation of RE   
    RE=(Syy-Syx*CE'-CE*Sxy-Syu*DE'-DE*Suy+CE*Sxuo*DE'+DE*Suxo*CE'+CE*Sxx*CE'+DE*Suuo*DE')/(T-1);
    [LRE,a]=eig(RE);
    i=find(((diag(a)<=0) | (imag(diag(a))~=0)));
    for j=1:length(i)
        a(i(j),i(j))=0;
    end
    RE=LRE*a*LRE';
    
 %-----------------------------      
    k=k+1;
end