function [x,P,PP,loglik,x_one_step,P_one_step,P_ant]=kalman_filter2s(y,u,A,B,C,D,Q,R,init_x,init_P);
%This function computes the standard steady-state Kalman filter
%for the model in the state space 

% x_{t+1} = A*x_{t} + B*u_{t} + w_{t}   var{w_(t)} = Q
% y_{t}   = C*x_{t} + D*u_{t} + v_{t}   var{v_(t)} = R

%in order to calculate:
%x(:,t)=E[X(:,t)|y(:,1:t),u(:,1:t)] t>=1
%P(:,:)=A*Var[X(:,t)|y(:,1:t),u(:,1:t)]*A + Q;
%PP=Cov[X(:,T),X(:,T-1)|y(:,1:T),u(:,1:T)] T the last time step 
%loglik ; loglik is the likelihood function

[os T]=size(y);
ss=size(A,1); %state space dimension

%Kalman filtering outputs
x=zeros(ss,T);
P=zeros(ss,ss);
PP=zeros(ss,ss);


%For t==1
x(:,1)=init_x; %initial state
[P,L,G,rr]=dare(A',C',Q,R); %solver for the Riccati equation: P, L=eig(A'-C'*G,eye(ss)), G=(A*K)', rr is the Frobenius norm of the residual matrix
K=A\G'; %Kalman gain matrix
S=C*P*C'+R; %innovation variance matrix
S_inv=inv(S);
loglik=log(det(S))*(T-1);

for t=2:T
    
    prevx=x(:,t-1); %prevx=E[X(:,t-1)|y(:,1:t-1)] 
    
    xpred=A*prevx+B*u(:,t-1); %xpred=E[X(:,t)|y(:,(1:t-1))]
    
    e=y(:,t)-C*xpred-D*u(:,t); %innovations 
    
    x(:,t)=xpred+K*e; %x=E[X(:,t)|y(:,1:t)]
    
    loglik=loglik+e'*S_inv*e;

end
loglik=-0.5*loglik;

%These matrices will be the initial matrices in the Kalman smoother filter
P_ant = (eye(ss)-K*C)*P;
PP=(eye(ss)-K*C)*A*P_ant; % PP=Cov[X(:,T),X(:,T-1)|y(:,1:T),u(:,1:T)]
x_one_step=A*x(:,T)+B*u(:,T); % x_one_step=E[X(:,T+1)|y(:,(1:T))]
P_one_step=P;   % P_one_step=Var[X(:,T+1)|y(:,(1:T))]
