function mc=run_models(y,r,ninputs)

%This function is the main function to estimate matrices in the state space model

% x_{t+1} = A*x_{t} + B*u_{t} + w_{t}   var{w_(t)} = Q
% y_{t}   = C*x_{t} + D*u_{t} + v_{t}   var{v_(t)} = R

%With this function we define the initial guess to call the EM algorithm to
%estimate the unknown matrices Q, R, C and D in the state space.
%Finally, we define the data structure mc with all the relevant information given in the paper.


%Inputs y, r, ninputs
%y are the data nxN (n=24 in the case of electricity prices)
%r is the order of the model
%ninputs: the number of inputs 1 or 5 in the paper.


%Outputs
%mc is a data structure with the most relevant information written in the paper.


epsilon=1e-4;
tol=1e-20;
niter=500;
[n,N]=size(y);

%Define the inputs of the problems, depending on the different kinds of days in the week. 
if (ninputs==0)
    ur=[0,0,0,0,0];
end

if (ninputs==1)
    ur=[1,1,1,1,1]; %Every day are equals
end
if (ninputs==3)
    ur=[1,0,0,0,0;0,1,1,1,0;0,0,0,0,1]; %There are three sets of days: Monday, Tuesday-Wednesday-Thursday, Friday
end
if (ninputs==5)
    ur=[1,0,0,0,0;0,1,0,0,0;0,0,1,0,0;0,0,0,1,0;0,0,0,0,1]; %The five days of the weeks are different
end
u=[];
for i=1:N/5
   u=[u,ur];
end

%Initial guess for matrices in the state space model
A=(1-epsilon)*eye(r);
B=zeros(r,length(u(:,1)));
Q=eye(r);

C=eye(24,r);
D=y(:,1)*ones(1,length(u(:,1)));
R=eye(24);

init_P=1e-10*eye(r);
init_x=zeros(r,1);

%Estimate the matrices Q, R, C, and D with the EM algorithm.
[A,B,C,D,Q,R,init_x,init_P,k,logvec]=em_algorithm(y,u,A,B,C,D,Q,R,init_x,init_P,tol,niter);

%Extract the most important information with the same notation as the paper.
mc.H = C;
mc.G = D;
mc.Q = Q;
mc.R = R;
mc.loglikelihood = logvec;

mc.np=12*(n+1)+0.5*r*(2*n-r+1)+n*ninputs;
mc.AIC=-2*logvec(end)+2*mc.np;  
mc.SIC=-2*logvec(end)+mc.np*log(N);



